#include "defines.h"

// Normal calculations
float crossresult[3];

extern external_objects extobj[NUMOBJS];
extern GLUquadricObj* gluObj;

// Calculate the cross product of triangles from RAW vertices
void cross_prod(int id, int i)
{
	float v1[3];
	float v2[3];
	float length;
	
	v1[0] = extobj[id].vx[i] - extobj[id].vx[i+1];
	v1[1] = extobj[id].vy[i] - extobj[id].vy[i+1];
	v1[2] = extobj[id].vz[i] - extobj[id].vz[i+1];
	v2[0] = extobj[id].vx[i+1] - extobj[id].vx[i+2];
	v2[1] = extobj[id].vy[i+1] - extobj[id].vy[i+2];
	v2[2] = extobj[id].vz[i+1] - extobj[id].vz[i+2];
	crossresult[0] = v1[1]*v2[2] - v2[1]*v1[2];
	crossresult[1] = v2[0]*v1[2] - v1[0]*v2[2];
	crossresult[2] = v1[0]*v2[1] - v2[0]*v1[1];

	length = sqrt(crossresult[0]*crossresult[0]
			+ crossresult[1]*crossresult[1]
			+ crossresult[2]*crossresult[2]);

	if(length == 0.0)
		length = 1.0;

	crossresult[0] /= length;
	crossresult[1] /= length;
	crossresult[2] /= length;
}

// Draw an object by ID (Each RAW file has an ID)
void drawextobj(int id, GLfloat factor)
{
	int i;
	
	for(i=0; i<extobj[id].numvertices; i+=3)
	{
		switch(extobj[id].colorid[i])
		{
			case 0:
				setMaterials(&redlaser, GL_FRONT_AND_BACK);
				break;
			case 1:
				setMaterials(&bluesolarpanels, GL_FRONT_AND_BACK);
				break;
			case 2:
				setMaterials(&gengrey, GL_FRONT_AND_BACK);
				break;
			case 3:
				setMaterials(&bluengines, GL_FRONT_AND_BACK);
				break;
			case 4:
				setMaterials(&Brass, GL_FRONT_AND_BACK);
				break;
			case 5:
				setMaterials(&Bronze, GL_FRONT_AND_BACK);
				break;
			case 6:
				setMaterials(&Polished_Bronze, GL_FRONT_AND_BACK);
				break;
			case 7:
				setMaterials(&Chrome, GL_FRONT_AND_BACK);
				break;
			case 8:
				setMaterials(&Copper, GL_FRONT_AND_BACK);
				break;
			case 9:
				setMaterials(&Polished_Copper, GL_FRONT_AND_BACK);
				break;
			case 10:
				setMaterials(&Gold, GL_FRONT_AND_BACK);
				break;
			case 11:
				setMaterials(&Polished_Gold, GL_FRONT_AND_BACK);
				break;
			case 12:
				setMaterials(&Pewter, GL_FRONT_AND_BACK);
				break;
			case 13:
				setMaterials(&Silver, GL_FRONT_AND_BACK);
				break;
			case 14:
				setMaterials(&Polished_Silver, GL_FRONT_AND_BACK);
				break;
			case 15:
				setMaterials(&Emerald, GL_FRONT_AND_BACK);
				break;
			case 16:
				setMaterials(&Jade, GL_FRONT_AND_BACK);
				break;
			case 17:
				setMaterials(&Obsidian, GL_FRONT_AND_BACK);
				break;
			case 18:
				setMaterials(&Pearl, GL_FRONT_AND_BACK);
				break;
			case 19:
				setMaterials(&Ruby, GL_FRONT_AND_BACK);
				break;
			case 20:
				setMaterials(&Turquoise, GL_FRONT_AND_BACK);
				break;
			case 21:
				setMaterials(&Black_Plastic, GL_FRONT_AND_BACK);
				break;
			case 22:
				setMaterials(&Black_Rubber, GL_FRONT_AND_BACK);
				break;
			default:
				 printf("Error: Screwed up model colorid label \"%d\"!?\n",
						 extobj[id].colorid[i]);
				 exit(1);
		}

		glBegin(GL_TRIANGLES);

			cross_prod(id,i);
			// We're calculating normals on the fly, ouch efficiency...
			glNormal3f(crossresult[0],crossresult[1],crossresult[2]);

			glVertex3f(extobj[id].vx[i]*factor,
					extobj[id].vy[i]*factor,
					extobj[id].vz[i]*factor);

			glVertex3f(extobj[id].vx[i+1]*factor,
					extobj[id].vy[i+1]*factor,
					extobj[id].vz[i+1]*factor);

			glVertex3f(extobj[id].vx[i+2]*factor,
					extobj[id].vy[i+2]*factor,
					extobj[id].vz[i+2]*factor);

		glEnd();
	}
}
